@@ -9,7 +9,7 @@
     Released under the GNU Public Licence, Version 2.
 */
 
-#define VERSION "2.4.1"
+#define VERSION "2.4.2"
 
 #include "ksymoops.h"
 #include <ctype.h>
@@ -28,6 +28,7 @@ char *path_objdump = "/usr/bin/objdump";    /* env KSYMOOPS_OBJDUMP */
 int debug = 0;
 int errors = 0;
 int warnings = 0;
+addr_t truncate_mask = ~(addr_t)1;
 
 SYMBOL_SET   ss_vmlinux;
 SYMBOL_SET   ss_ksyms_base;
@@ -72,6 +73,8 @@ static void usage(void)
 	   "\t[-e] [--endian-swap]\t\tToggle endianess of code bytes\n"
 	   "\t[-x] [--hex]\t\t\tHex or decimal toggle\n"
 	   "\t[-1] [--one-shot]\t\tOne shot toggle (exit after first Oops)\n"
+	   "\t[-i] [--ignore-insmod-path]\tIgnore path from __insmod symbols, toggle\n"
+	   "\t[-I] [--ignore-insmod-all]\tIgnore all __insmod symbols, toggle\n"
 	   "\t[-d] [--debug]\t\t\tIncrease debug level by 1\n"
 	   "\t[-h] [--help]\t\t\tPrint help text\n"
 	   "\t[-t target] [--target=target]\tTarget of oops log\n"
@@ -267,6 +270,9 @@ static void parse(int argc, char **argv, struct options *options, int *spec_h)
 	{"endian-swap", 0, 0, 'e'},
 	{"hex", 0, 0, 'x'},
 	{"one-shot", 0, 0, '1'},
+	{"ignore-insmod-path", 0, 0, 'i'},
+	{"ignore-insmod-all", 0, 0, 'I'},
+	{"truncate", 1, 0, 'T'},
 	{"debug", 0, 0, 'd'},
 	{"help", 0, 0, 'h'},
 	{"target", 1, 0, 't'},
@@ -277,7 +283,7 @@ static void parse(int argc, char **argv, struct options *options, int *spec_h)
     int c, i, some_spec = 0;
     char *p, *before;
 
-    while ((c = getopt_long(argc, argv, "v:Vk:Kl:Lo:Om:Ms:e1xShdt:a:",
+    while ((c = getopt_long(argc, argv, "v:Vk:Kl:Lo:Om:Ms:e1xSiIT:hdt:a:",
 	    long_opts, NULL)) != EOF) {
 	if (c != 'd')
 	    DEBUG(1, "'%c' '%s'", c, optarg);
@@ -358,6 +364,22 @@ static void parse(int argc, char **argv, struct options *options, int *spec_h)
 	case 'S':
 	    options->short_lines = !options->short_lines;
 	    break;
+	case 'i':
+	    options->ignore_insmod_path = !options->ignore_insmod_path;
+	    break;
+	case 'I':
+	    options->ignore_insmod_all = !options->ignore_insmod_all;
+	    break;
+	case 'T':
+	    options->truncate = atoi(optarg);
+	    if (options->truncate >= 8*sizeof(addr_t) || options->truncate <= 0) {
+		options->truncate = 0;
+		truncate_mask = ~(addr_t)1;
+	    }
+	    else {
+		truncate_mask = (~(addr_t)1) >> (8*sizeof(addr_t)-options->truncate);
+	    }
+	    break;
 	case 'h':
 	    usage();
 	    ++*spec_h;
@@ -458,6 +480,22 @@ static void parse(int argc, char **argv, struct options *options, int *spec_h)
 	printf("%s -1", before);
 	before = "";
     }
+    if (options->ignore_insmod_path) {
+	printf("%s -i", before);
+	before = "";
+    }
+    if (options->ignore_insmod_all) {
+	printf("%s -I", before);
+	before = "";
+    }
+    if (!*before)
+	printf("\n");
+
+    before = "    ";
+    if (options->truncate) {
+	printf("%s -T %d", before, options->truncate);
+	before = "";
+    }
     if (!*before)
 	printf("\n");
 
@@ -479,7 +517,7 @@ static void parse(int argc, char **argv, struct options *options, int *spec_h)
     if (!some_spec) {
 	/* special warning, no procname */
 	++warnings;
-	WARNING_E("%s", 
+	WARNING_E("%s",
 "Warning: You did not tell me where to find symbol information.  I will\n"
 "assume that the log matches the kernel and modules that are running\n"
 "right now and I'll use the default options above for symbol resolution.\n"
@@ -537,8 +575,8 @@ void read_symbol_sources(const OPTIONS *options)
 {
     int i;
 
-    read_vmlinux(options->vmlinux);
-    read_ksyms(options->ksyms);
+    read_vmlinux(options);
+    read_ksyms(options);
     /* No point in reading modules unless ksyms shows modules loaded */
     if (ss_ksyms_modules) {
 	expand_objects(options);
@@ -549,11 +587,11 @@ void read_symbol_sources(const OPTIONS *options)
 	printf("No modules in ksyms, skipping objects\n");
     /* No point in reading lsmod without ksyms */
     if (ss_ksyms_modules || ss_ksyms_base.used)
-	read_lsmod(options->lsmod);
+	read_lsmod(options);
     else if (options->lsmod)
 	printf("No ksyms, skipping lsmod\n");
-    read_system_map(options->system_map);
-    merge_maps(options->save_system_map, options);
+    read_system_map(options);
+    merge_maps(options);
 }
 
 int main(int argc, char **argv)
@@ -574,6 +612,9 @@ int main(int argc, char **argv)
 	0,	/* endianess */
 	1,	/* hex */
 	0,	/* one_shot */
+	0,	/* ignore_insmod_path */
+	0,	/* ignore_insmod_all */
+	0,	/* truncate */
 	NULL,   /* target */
 	NULL,   /* architecture */
 	0	/* address_bits */
