@@ -113,7 +113,7 @@ static int Oops_write_bfd_data(const bfd *ibfd, bfd *obfd,
 	Oops_bfd_perror("make_section");
 	return(0);
     }
-    if (!bfd_set_section_flags(obfd, osec, 
+    if (!bfd_set_section_flags(obfd, osec,
 	bfd_get_section_flags(ibfd, isec))) {
 	Oops_bfd_perror("set_section_flags");
 	return(0);
@@ -514,10 +514,41 @@ static void Oops_set_eip(const char *value, addr_t *eip, SYMBOL_SET *ss,
     add_symbol_n(ss, *eip, 'E', 1, eip_name);
 }
 
+/* Look for the sysrq-t (show_task) process line and extract the command name */
+static int Oops_set_task_name(const char *line, char ***string, int string_max, SYMBOL_SET *ss)
+{
+    int i;
+    char id[64];
+    static regex_t     re_Oops_set_task_name;
+    static regmatch_t *re_Oops_set_task_name_pmatch;
+    static const char procname[] = "Oops_set_task_name";
+
+    /* Task header from show_task contains (L-TLB) or (NOTLB), the first field
+     * is the command name.
+     */
+    RE_COMPILE(&re_Oops_set_task_name,
+	    "^([^ ]+) +[^ ] *([^ ]+).*\\((L-TLB|NOTLB)\\)",
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_set_task_name_pmatch);
+
+    i = regexec(&re_Oops_set_task_name, line, re_Oops_set_task_name.re_nsub+1,
+	re_Oops_set_task_name_pmatch, 0);
+    DEBUG(4, "regexec %d", i);
+    if (i)
+	return(0);
+    re_string_check(re_Oops_set_task_name.re_nsub+1, string_max, procname);
+    re_strings(&re_Oops_set_task_name, line, re_Oops_set_task_name_pmatch, string);
+    snprintf(id, sizeof(id), "Proc;  %s", (*string)[1]);
+    add_symbol_n(ss, 0, 'I', 1, id);
+    return(1);
+}
+
 /******************************************************************************/
 /*                     Start architecture sensitive code                      */
 /******************************************************************************/
 
+static regex_t     re_Oops_cris_regs;
+static regmatch_t *re_Oops_cris_regs_pmatch;
 static regex_t     re_Oops_i370_regs;
 static regmatch_t *re_Oops_i370_regs_pmatch;
 static int         s390_reg_num = -1;
@@ -535,9 +566,10 @@ static addr_t Oops_truncate_address(addr_t address, const OPTIONS *options)
 	if (strstr(options->target, "i370") ||
 	    strstr(options->target, "s390"))
 	    u32 &= 0x7fffffff;  /* Really 31 bit addressing on i370/s390 */
-	return(u32);
+	address = u32;
     }
-    return(address);	/* 0, 64, anything else */
+    address &= truncate_mask;
+    return(address);
 }
 
 /* Return a string to represent the eip, depending on the target. */
@@ -549,6 +581,7 @@ static const char *Oops_arch_to_eip(const OPTIONS *options)
 	return("EIP");
     if (strstr(options->target, "alpha") ||
 	strstr(options->target, "arm")   ||
+	strstr(options->target, "cris")  ||
 	strstr(options->target, "m68k")  ||
 	strstr(options->target, "mips"))
 	    return("PC");
@@ -585,6 +618,8 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
     static regmatch_t *re_Oops_eip_kdb_pmatch;
     static regex_t     re_Oops_eip_ia64;
     static regmatch_t *re_Oops_eip_ia64_pmatch;
+    static regex_t     re_Oops_eip_cris;
+    static regmatch_t *re_Oops_eip_cris_pmatch;
     static regex_t     re_Oops_eip_other;
     static regmatch_t *re_Oops_eip_other_pmatch;
     static const char procname[] = "Oops_eip";
@@ -728,6 +763,26 @@ static char *Oops_eip(const char *line, char ***string, int string_max,
 	    return((*string)[5]);
     }
 
+    /* Oops 'IRP' line for CRIS: "^IRP: " unbracketed address.
+     * Keep spacing optional.
+     */
+    RE_COMPILE(&re_Oops_eip_cris,
+	    "^ *IRP *: *"
+	    UNBRACKETED_ADDRESS,
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_eip_cris_pmatch);
+
+    i = regexec(&re_Oops_eip_cris, line, re_Oops_eip_cris.re_nsub+1,
+	re_Oops_eip_cris_pmatch, 0);
+    DEBUG(4, "regexec cris %d", i);
+    if (i == 0) {
+	re_string_check(re_Oops_eip_cris.re_nsub+1, string_max,
+	    procname);
+	re_strings(&re_Oops_eip_cris, line, re_Oops_eip_cris_pmatch,
+	    string);
+	return((*string)[re_Oops_eip_cris.re_nsub]);
+    }
+
     /* Oops 'EIP:' line for other architectures */
     RE_COMPILE(&re_Oops_eip_other,
 		"^("
@@ -871,7 +926,7 @@ static void Oops_set_ia64_b0(const char *line, char ***string, int string_max, S
     static const char procname[] = "Oops_ia64_b0";
 
     /* Oops 'b0' line for ia64, b0 : unbracketed address, may be
-     * repeated for b6, b7. 
+     * repeated for b6, b7.
      */
     RE_COMPILE(&re_Oops_ia64_b0,
 	    "^b[0-7] *: *"
@@ -1068,7 +1123,7 @@ static const char *Oops_trace(const char *line)
     static const char procname[] = "Oops_trace";
 
     /* Oops 'Trace' lines.  The ikd patch adds "(nn)" for the stack frame size
-     * after each trace entry, it should appear on the same line as the 
+     * after each trace entry, it should appear on the same line as the
      * bracketed address.  Alas users have been known to split lines so "(nn)"
      * appears at the start of the continuation line, look for that as well.
      * Life would be so much easier without users.
@@ -1209,14 +1264,77 @@ static void Oops_trace_line(const char *line, const char *p, SYMBOL_SET *ss)
     re_strings_free(pregex, &string);
 }
 
+/* Decide if the line contains CRIS registers.  Returns first register name. */
+static const char *Oops_cris_regs(const char *line)
+{
+    int i;
+    static const char procname[] = "Oops_cris_regs";
+
+    /* Note that most of the register dump, the r0: to oR10: lines, are
+       handled by Oops_i370_regs.  */
+    RE_COMPILE(&re_Oops_cris_regs,
+	    "^(IRP|SRP|D?CCR|USP|MOF): *" UNBRACKETED_ADDRESS,
+	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
+	&re_Oops_cris_regs_pmatch);
+
+    i = regexec(&re_Oops_cris_regs, line, re_Oops_cris_regs.re_nsub+1,
+	re_Oops_cris_regs_pmatch, 0);
+    DEBUG(4, "regexec %d", i);
+    if (i == 0)
+	return(line + re_Oops_cris_regs_pmatch[0].rm_so);
+    return(NULL);
+}
+
+/* Process a CRIS register line, extract addresses */
+static void Oops_set_cris_regs(const char *line, const char *p,
+			       SYMBOL_SET *ss, const OPTIONS *options)
+{
+    addr_t reg;
+    char regname[7];
+    char **string = NULL;
+    static const char procname[] = "Oops_set_cris_regs";
+
+    /* Loop over CRIS register names and unbracketed addresses.  */
+    while (1) {
+	if (regexec(&re_Oops_cris_regs, p, re_Oops_cris_regs.re_nsub+1,
+	    re_Oops_cris_regs_pmatch, 0) == 0) {
+	    re_strings(&re_Oops_cris_regs, p, re_Oops_cris_regs_pmatch, &string);
+	    reg = hexstring(string[2]);		/* contents */
+	    if (errno) {
+		ERROR(" Invalid hex value in CRIS register line, ignored - '%s'",
+		      string[2]);
+		perror(prefix);
+		reg = 0;
+	    }
+	    else {
+	      reg = Oops_truncate_address(reg, options);
+	      strcpy(regname, ">>");
+	      strncpy(regname+2, string[1], sizeof(regname)-3);
+	      strcat(regname, ";");
+	      add_symbol_n(ss, reg, 'R', 1, regname);
+	    }
+	    p += re_Oops_cris_regs_pmatch[0].rm_eo;
+	}
+	else
+	    break;
+    }
+
+    if (*p)
+	WARNING("garbage '%s' at end of CRIS register line ignored", p);
+    re_strings_free(&re_Oops_cris_regs, &string);
+}
+
 /* Decide if the line contains i370 registers.  Returns first register name. */
 static const char *Oops_i370_regs(const char *line)
 {
     int i;
     static const char procname[] = "Oops_i370_regs";
 
+    /* Matching "oR10: unbracketed address" enables us to also match a
+       CRIS register dump for the lines that overlap with the i370
+       register dump format.  */
     RE_COMPILE(&re_Oops_i370_regs,
-	    "^(r[0-9]{1,2}): *" UNBRACKETED_ADDRESS,	/* 1, 2 */
+	    "^(o?r[0-9]{1,2}): *" UNBRACKETED_ADDRESS,	/* 1, 2 */
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_i370_regs_pmatch);
 
@@ -1252,11 +1370,9 @@ static void Oops_set_i370_regs(const char *line, const char *p,
 	    reg = Oops_truncate_address(reg, options);
 	    if (reg > 1024) {
 		strcpy(regname, ">>");
-		strncpy(regname+2, string[1], sizeof(regname-2));
-		if (regname[4] == '\0')
-		    strcpy(regname+4, " ;");
-		else
-		    strcpy(regname+5, ";");
+		strncpy(regname+2, string[1], sizeof(regname)-2);
+		if (strlen(regname) < sizeof(regname)-1)
+		    strcat(regname, ";");
 		add_symbol_n(ss, reg, 'R', 1, regname);
 	    }
 	    p += re_Oops_i370_regs_pmatch[0].rm_eo;
@@ -1293,7 +1409,7 @@ static const char *Oops_s390_regs(const char *line)
     if (i == 0) {
 	s390_reg_num = 0;  /* Start with R0 */
 	return(line + re_Oops_s390_regs_pmatch[0].rm_eo);
-    }	
+    }
     else if (s390_reg_num >= 0) /* Header already seen */ {
 	i = regexec(&re_unbracketed_address, line,
 	    re_unbracketed_address.re_nsub+1,
@@ -1302,7 +1418,7 @@ static const char *Oops_s390_regs(const char *line)
 	if (i == 0)
 	    return(line + re_unbracketed_address_pmatch[0].rm_so);
     }
-    s390_reg_num = -1;  /* Not processing s390 registers */	
+    s390_reg_num = -1;  /* Not processing s390 registers */
     return(NULL);
 }
 
@@ -1410,7 +1526,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
      * the patterns into two groups, one set must appear at the start of the
      * line, the other set can appear anywhere.
      *
-     * Lots of bracketed subexpressions really slows regex down.  Only use 
+     * Lots of bracketed subexpressions really slows regex down.  Only use
      * brackets where absolutely necessary.  Examples are where you want to
      * extract matching substrings such as stack, call trace etc.  Or where
      * you have suboptions within a string, like the sparc die_if_kernel text.
@@ -1516,7 +1632,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
 
 
     /* order does not matter from here on */
-    
+
     /* various */   "|Process .*stackpage="
     /* various */   "|Code *: "
     /* various */   "|Kernel panic"
@@ -1641,6 +1757,9 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* ia64 */	    "|General Exception"
     /* any other ia64 lines to print? */
 
+    /* CRIS */	    "|IRP *: "
+    /* any other cris lines to print? */
+
 		")",
 	REG_NEWLINE|REG_EXTENDED|REG_ICASE,
 	&re_Oops_print_s_pmatch);
@@ -1695,7 +1814,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
 	 * it is a filename.  Look for '.c:' before the first space and ignore
 	 * if found.  Also ad1848, look for '/' before first space.  Do not look
 	 * for ':' on its own, arm does "address: stack values" :(.
-	 * 
+	 *
 	 * Ambiguity of ambiguities, saith the preacher; all is ambiguity.
 	 */
 	const char *p1, *p2;
@@ -1731,6 +1850,8 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* various */   "|die_if_kernel"  /* ditto */
     /* various */   "|NMI "
     /* various */   "|BUG "
+    /* various */   "|\\(L-TLB\\)"
+    /* various */   "|\\(NOTLB\\)"
 
     /* alpha */     "|\\([0-9]\\): Oops "
     /* alpha */     "|: memory violation"
@@ -1799,7 +1920,7 @@ static const char *Oops_code(const char *line, char ***string, int string_max)
 		": +"
 		"("				/*  4 */
 		  "(general protection.*)"
-		  "|(Bad EIP value.*)"
+		  "|(Bad E?IP value.*)"
 		  "|(<[0-9]+>)"
 		  "|(([<(]?[0-9a-fA-F]+[>)]? +)+[<(]?[0-9a-fA-F]+[>)]?)"
 		")"
@@ -1857,7 +1978,7 @@ static void Oops_decode(const unsigned char* code_text, addr_t eip,
     if (!(file = Oops_code_to_file(code, CODE_SIZE, ibfd, options)))
 	return;
     /* objdump the pseudo object */
-    if (!(f = Oops_objdump(file, options)))   
+    if (!(f = Oops_objdump(file, options)))
 	return;
     while (fgets_local(&line, &size, f, procname)) {
 	DEBUG(2, "%s", line);
@@ -1889,8 +2010,8 @@ static void Oops_format(const SYMBOL_SET *ss_format, OPTIONS *options)
     printf("\n");
     for (s = ss_format->symbol, i = 0; i < ss_format->used; ++i, ++s) {
 	/* For type C data, print Code:, address, map, "name" (actually
-	 * the text of an objdump line).  For other types print name,
-	 * address, map.
+	 * the text of an objdump line).  For type I (id) print the string.
+	 * For other types print name, address, map.
 	 */
 	if (s->type == 'C') {
 	    eye_catcher = "";
@@ -1908,6 +2029,9 @@ static void Oops_format(const SYMBOL_SET *ss_format, OPTIONS *options)
 		    map_address(&ss_merged, s->address, options),
 		    s->name, eye_catcher);
 	}
+	else if (s->type == 'I') {
+	    printf("%s\n", s->name);
+	}
 	else {
 	    printf("%s %s %s",
 		s->name,
@@ -2036,7 +2160,11 @@ int Oops_read(OPTIONS *options)
 		    Oops_set_i370_regs(text, start, &ss_format, options);
 		if ((start = Oops_s390_regs(text)))
 		    Oops_set_s390_regs(text, start, &ss_format, options);
+		if ((start = Oops_cris_regs(text)))
+		    Oops_set_cris_regs(text, start, &ss_format, options);
 		Oops_set_ia64_b0(text, &string, MAX_STRINGS, &ss_format);
+		if (Oops_set_task_name(text, &string, MAX_STRINGS, &ss_format))
+		    Oops_set_eip(string[2], &eip, &ss_format, me, ibfd, options);
 		if ((start = Oops_code(text, &string, MAX_STRINGS))) {
 		    /* In case no EIP line was seen */
 		    Oops_set_default_ta(me, ibfd, options);
