@@ -72,10 +72,11 @@ SYMBOL *find_symbol_name(const SYMBOL_SET *ss, const char *symbol, int *start)
  * NOTE: Assumes that ss is sorted by address.
  */
 static SYMBOL *find_symbol_address(const SYMBOL_SET *ss,
-				   const addr_t address, int *start)
+				   addr_t address, int *start)
 {
     int i;
     SYMBOL *s;
+    address &= truncate_mask;
     for (i = start ? *start : 0, s = ss->symbol+i; i < ss->used; ++i, ++s) {
 	if (address > s->address)
 	    continue;
@@ -85,7 +86,7 @@ static SYMBOL *find_symbol_address(const SYMBOL_SET *ss,
 		/* more than one symbol on the same address, skip the ones that
 		 * came from modutils assist.
 		 */
-		continue; 
+		continue;
 	    if (start)
 		*start = i;
 	    return(s);
@@ -99,7 +100,7 @@ static SYMBOL *find_symbol_address(const SYMBOL_SET *ss,
 }
 
 /* Add a symbol to a symbol set, address in binary */
-int add_symbol_n(SYMBOL_SET *ss, const addr_t address,
+int add_symbol_n(SYMBOL_SET *ss, addr_t address,
 		 const char type, const char keep, const char *symbol)
 {
     int i;
@@ -109,6 +110,7 @@ int add_symbol_n(SYMBOL_SET *ss, const addr_t address,
     static regmatch_t *re_symbol_ver_pmatch;
     static const char procname[] = "add_symbol_n";
 
+    address &= truncate_mask;
     /* Strip out any trailing symbol version _R.*xxxxxxxx. */
     RE_COMPILE(&re_symbol_ver,
 	"^(.*)_R.*[0-9a-fA-F]{8,}$",
@@ -173,7 +175,7 @@ int add_symbol(SYMBOL_SET *ss, const char *address, const char type,
 }
 
 /* Map an address to symbol, offset and length, address in binary */
-char *map_address(const SYMBOL_SET *ss, const addr_t address,
+char *map_address(const SYMBOL_SET *ss, addr_t address,
 		  const OPTIONS *options)
 {
     int i = 0, l;
@@ -182,11 +184,12 @@ char *map_address(const SYMBOL_SET *ss, const addr_t address,
     static int size = 0;
     static const char procname[] = "map_address_n";
 
+    address &= truncate_mask;
     DEBUG(3, "%s %s", ss->source, format_address(address, options));
     s = find_symbol_address(ss, address, &i);
     if (!s && --i >= 0)
 	s = ss->symbol+i;   /* address is between s and s+1 */
-	    
+
     /* Extra map text is always < 100 bytes */
     if (s) {
 	l = strlen(s->name) + 100;
